import pytest
import platform
import datetime

import os

from trash_manager import TrashItem


@pytest.mark.skipif(platform.system() != "Linux", reason="Function specific to linux")
def test__LINUX__TrashItem_for_trashed_file(tmp_path):
    # Prepare fake data
    original_filepath = "/tmp/my_file"
    deletion_date = "2024-08-07T01:53:48"
    data = f"""
[Trash Info]
Path={original_filepath}
DeletionDate={deletion_date}
""".encode()

    # Prepare fake trash
    os.mkdir(os.path.join(tmp_path, "Trash"))
    os.mkdir(os.path.join(tmp_path, "Trash", "files"))
    os.mkdir(os.path.join(tmp_path, "Trash", "info"))

    # Create a fake trashed file
    trashed_filepath = os.path.join(tmp_path, "Trash", "files", "from_fileinfo")
    open(trashed_filepath, 'w').close()

    # Create a fake trashinfo file
    trashinfo_filepath = os.path.join(tmp_path, "Trash", "info", "from_fileinfo.trashinfo")
    with open(trashinfo_filepath, 'wb') as f:
        f.write(data)

    # Test the alternative constructor
    ti = TrashItem.for_trashed_file(trashed_filepath)
    assert ti.trashinfo_path == trashinfo_filepath
    assert ti.trashed_path == trashed_filepath
    assert ti.original_path == original_filepath
    assert ti.deletion_date == datetime.datetime.fromisoformat(deletion_date)


def test_TrashItem_recover(tmp_path):
    # Prepare fake data
    original_filepath = f"{tmp_path}/my_file"
    deletion_date = "2024-08-07T01:53:48"
    data = f"""
    [Trash Info]
    Path={original_filepath}
    DeletionDate={deletion_date}
    """.encode()

    # Prepare fake trash
    os.mkdir(os.path.join(tmp_path, "Trash"))
    os.mkdir(os.path.join(tmp_path, "Trash", "files"))
    os.mkdir(os.path.join(tmp_path, "Trash", "info"))

    # Create a fake trashed file
    trashed_filepath = os.path.join(tmp_path, "Trash", "files", "my_file")
    open(trashed_filepath, 'w').close()

    # Create a fake trashinfo file
    trashinfo_filepath = os.path.join(tmp_path, "Trash", "info", "my_file.trashinfo")
    with open(trashinfo_filepath, 'wb') as f:
        f.write(data)

    ti = TrashItem.for_trashed_file(trashed_filepath)
    ti.recover()

    assert os.path.exists(original_filepath)
    assert not os.path.exists(trashed_filepath)
    assert not os.path.exists(trashinfo_filepath)



import os
from pathlib import Path
import platform
from .trash_item import TrashItem


class ITrashManager:
    USER_TRASH = ""

    def list(self, trash_path: str = "") -> list[TrashItem]:
        raise NotImplementedError("Trash Manager doesn't implement this method")


if platform.system() == "Linux":
    class TrashManager(ITrashManager):

        @property
        def USER_TRASH(self) -> str:
            prefix = os.environ.get("XDG_DATA_HOME", "")
            if prefix == "":
                prefix = os.path.expandvars("$HOME/.local/share")
            return os.path.join(prefix, "Trash")

        def list(self, trash_path: str = "") -> list[TrashItem]:
            if trash_path == "":
                trash_path = self.USER_TRASH
            trash_path = Path(trash_path)

            files_path = trash_path / "files"
            trash_data = []

            if files_path.exists():
                for filename in files_path.iterdir():
                    ti = TrashItem.for_trashed_file(str(filename))
                    if ti is not None:
                        trash_data.append(ti)
            return trash_data

else:
    class TrashManager(ITrashManager):
        def __init__(self):
            raise NotImplementedError("TrashManager for this OS is not implemented")



import pytest
import platform

import os

from trash_manager import TrashManager


@pytest.mark.skipif(platform.system() != "Linux", reason="Function specific to linux")
def test__LINUX__TrashManager_USER_TRASH():
    tm = TrashManager()

    if "XDG_DATA_HOME" in os.environ:
        del os.environ["XDG_DATA_HOME"]

    expected_path = os.path.expandvars("$HOME/.local/share/Trash")
    assert tm.USER_TRASH == expected_path

    os.environ["XDG_DATA_HOME"] = "/tmp/home"
    assert tm.USER_TRASH == "/tmp/home/Trash"
    del os.environ["XDG_DATA_HOME"]


def test_TrashManager_list():
    tm = TrashManager()
    assert type(tm.list()) == list



from __future__ import annotations
import datetime
import platform
from pathlib import Path


class ITrashItem:
    def __init__(self, original_path: str, deletion_date: datetime.datetime):
        self.original_path = original_path
        self.deletion_date = deletion_date


if platform.system() == "Linux":
    class TrashItem(ITrashItem):
        def __init__(self, original_path: str, deletion_date: datetime.datetime, trashed_path: str, trashinfo_path: str):
            ITrashItem.__init__(self, original_path, deletion_date)
            self.trashed_path = trashed_path
            self.trashinfo_path = trashinfo_path

        @staticmethod
        def for_trashed_file(path: str) -> TrashItem:
            def find_tag(start: int, tag: str):
                pos = content.find(tag, start)
                if pos == -1: return -1, -1
                end_pos = content.find(b'\n', pos)
                return (pos + len(tag), end_pos) if end_pos != -1 else (-1, -1)

            trash_info_tag = b"[Trash Info]"
            path_tag = b"Path="
            deletion_date_tag = b"DeletionDate="

            path = Path(path)
            trashinfo_path = path.parent.parent / "info" / (path.name + ".trashinfo")
            with open(str(trashinfo_path), 'rb') as f:
                content = f.read()

            trash_info_pos = content.find(trash_info_tag)
            if trash_info_pos == -1:
                return None

            path_pos, path_end_pos = find_tag(trash_info_pos, path_tag)
            deletion_date_pos, deletion_date_end_pos = find_tag(trash_info_pos, deletion_date_tag)
            if path_pos == -1 or deletion_date_pos == -1:
                return None

            path_value = content[path_pos:path_end_pos].decode()
            deletion_date_value = datetime.datetime.fromisoformat(content[deletion_date_pos:deletion_date_end_pos].decode())
            return TrashItem(path_value, deletion_date_value, str(path), str(trashinfo_path))

        def recover(self, recover_to: str = ""):
            if recover_to == "":
                recover_to = self.original_path
            recover_to = Path(recover_to)
            trashed_file = Path(self.trashed_path)
            trashinfo_file = Path(self.trashinfo_path)

            if recover_to.exists():
                raise FileExistsError("A file exists in the trashed file's place")

            trashed_file.rename(str(recover_to))
            trashinfo_file.unlink(True)
